/********************************************************************
 * Q[C
 *******************************************************************/

#define GLOBAL_VALUE_DEFINE

#include <stdlib.h>
#include "s1c17702.h"
#include "game.h"
#include "bitmap.h"
#include "type.h"
#include "key.h"
#include "interrupt.h"

//LN^f[^
extern UInt8 _my_machine[];
extern UInt8 _piller_data[];
extern UInt8 _ball_data[];
extern UInt8 _title[];
extern UInt16 _hr_data[];
extern UInt16 _ready_font[];
extern UInt16 _level_font[];
extern UInt16 _pause_font[];
extern UInt16 _num_font[];
extern UInt16 _score_font[];
extern UInt16 _hi_font[];
extern UInt16 _over_font[];
extern UInt8 _bomb_data[];
extern UInt16 _bonus_font[];

//vg^Cv
static void init_game( void );
static void game( void );
static void init_machine( void );
static void move_machine( void );
static void draw_machine( void );
static void title( void );
static void init_level( void );
static void init_piller( void );
static void create_new_piller( int p_num );
static void draw_piller( int p_num );
static void disp_piller( void );
static void move_piller( void );
static Boolean is_crash( int ex );
static void init_horizon( void );
static void draw_horizon( void );
static void update_horizon( void );
static void level_up( void );
static void print_ready( void );
static void start_machine( void );
static void init_ready( void );
static void print_level( void );
static void disp_num( int  x, int  y, UInt16 num, int d );
static void add_score( void );
static void print_pause( void );
static void pause( void );
static void disp_score( int pos );
static void set_bomb( void );
static void disp_bomb( void );
static void print_over( void );
static void update_hiscore( void );
static void start_count( void );
static void flash( void );
static void flash_led( void );
static void set_flash( void );
static void set_flash_led( void );
static void bonus( void );

/************************************************
 *   _}CY
 ***********************************************/
static void
randomize(void)
{
  UInt16 seed = T16_TC0_TC;
  // ^C}[𗐐̎
  srand( seed );
}

/************************************************
 * LCD荞݃[`Ă΂
 * Q[{́@t[Ɏs
 ***********************************************/
void
enter_frame( void )
{
  game();
}

static void
game( void )
{
  get_key();

  switch( _game_state.mode )
    {
  case GM_TITLE:
    clear_screen();
    title();
    break;

  case GM_READY:
    draw_horizon();
    update_horizon();
    print_ready();
    start_machine();
//    move_machine();
    draw_machine();
    break;

  case GM_INTRO:
    draw_horizon();
    update_horizon();
    print_level();
    move_machine();
    draw_machine();
    start_count();
    break;

  case GM_PLAY:
    //clear_screen();
    draw_horizon();

    if( ! _game_state.pause_f )
      {
        update_horizon();
        move_piller();
        move_machine();
        add_score();
        level_up();
      }
    disp_piller();
    draw_machine();
    print_level();
    print_pause();
    bonus();
    pause();
    flash_led();
    break;

  case GM_OVER:
    draw_horizon();
    update_horizon();
    move_piller();
    disp_piller();
    disp_bomb();
    flash();
    print_over();
    break;
    }

  disp_screen();
}

/************************************************
 * ^Cg\
 *
 ***********************************************/
static void
title( void )
{
  int i, j, line;
  UInt8 *screen = (UInt8 *) _vram + (SCREEN_WIDTH + SCREEN_WIDTH_BLANK);
  UInt8 *src = (UInt8 *) _title;

  if( ! (--_pre_title.interval) )
    {
      _pre_title.interval = DEF_TITLE_INTERVAL;
      (UInt16) _pre_title.blink_f ^= 1;
    }

  line = 4;
  if( _pre_title.blink_f )// \C
    {
      line--;
    }

  for( j = 0; j < line; j++ )
    {
      for( i = 0; i < LCD_WIDTH; i++ )
        {
          *screen++ = *src++;
        }
      screen += (SCREEN_WIDTH_BLANK * 2);
    }

  if( _keyf & KEY_RIGHT || _keyf & KEY_LEFT )
    {
      _game_state.mode = GM_READY;
      init_game();//Q[
    }
}

/************************************************
 * @ړ
 *
 ***********************************************/
static void
move_machine( void )
{
  if( _key & KEY_RIGHT )
    {
      _machine.x++;
      if( _machine.x > SCREEN_WIDTH_BLANK + LCD_WIDTH - MACHINE_WIDTH )
        {
          _machine.x = SCREEN_WIDTH_BLANK + LCD_WIDTH - MACHINE_WIDTH;
        }
    }

  if( _key & KEY_LEFT )
    {
      _machine.x--;
      if( _machine.x < SCREEN_WIDTH_BLANK )
        {
          _machine.x = SCREEN_WIDTH_BLANK;
        }
    }
}

/************************************************
 * @\
 *
 ***********************************************/
static void
draw_machine( void )
{
  draw_bitmap( _machine.x, _machine.y, MACHINE_WIDTH,
      _my_machine );
}

/************************************************
 * Q[
 *
 ***********************************************/
static void
init_game( void )
{
  randomize();

  init_horizon();
  init_machine();
  init_level();
  init_piller();

  init_ready();

  _game_state.pause_f = false;
  _game_state.score = 0;

  _flash.time = 0;
}

/************************************************
 * X^[g
 *
 ***********************************************/
static void
init_ready( void )
{
  // _Őݒ
  _blink.blink_f = true;
  _blink.interval = DEF_BLINK_INTERVAL;
  _blink.time = DEF_BLINK_TIME;

  _start.wait = DEF_START_WAIT  * 10;
}



/************************************************
 * @
 *
 ***********************************************/
static void
init_machine( void )
{
  _machine.x = DEF_MACHINE_X;
  _machine.y = DEF_MACHINE_Y;
}

/************************************************
 * x
 *
 ***********************************************/
static void
init_level( void )
{
  _game_state.level = 0;
  _game_state.piller_num = START_PILLER_NUM;
  _game_state.lv_count = DEF_LV_COUNT;
}

/************************************************
 * n
 *
 ***********************************************/
static void
init_horizon( void )
{
  _horizon.no = 0;
  _horizon.wait = DEF_HR_WAIT;
}

/************************************************
 *
 *
 ***********************************************/
static void
init_piller( void )
{
  int i;

  for( i = 0; i < MAX_PILLER; i++ )
    {
      create_new_piller( i );
    }
  // EFCgݒiȂƑj
  _game_state.piller_wait = DEF_PILLER_WAIT;
}

/************************************************
 *VZbg
 *in: ݂̒
 ***********************************************/
static void
create_new_piller( int p_num )
{
  UInt8 fix;

  //LN^ԍ + EFCg˂
    _piller[ p_num ].num = 0xffff - ( RND(256) & 63 );

  // XW:@7fff-7f80@́@8000-807F
  fix = RND(256) - 128;

  if( (Int8) fix < 0 )
    _piller[ p_num ].x = 0x7f00 | fix;
  else
    _piller[ p_num ].x = 0x8000 | fix;

  // :@0 @1@{[
  _piller[ p_num ].kind = RND( 4 ) ? 0 : 1;
}

/************************************************
 *\
 *in: ݂̒
 *
 ***********************************************/
static void
draw_piller( int p_num )
{
  int i, j;
  //Wϊĕ\
  UInt16 x = ( (UInt16) _piller[ p_num ].x >> 8 ) - ( 164 - 80 + 4 ) + SCREEN_WIDTH_BLANK - PILLER_WIDTH;//8bit VRAM̍Wɕϊ
  UInt8 *screen = (UInt8 *) _vram + SCREEN_WIDTH + x;
  UInt8 *src = _piller[ p_num ].kind ? (UInt8 *)&_ball_data[ _piller[ p_num ].num * 32 ] : (UInt8 *)&_piller_data[ _piller[ p_num ].num * 32 ];

  if( _piller[ p_num ].num >= MAX_PILLER_SIZE ) return;


  for( j = 0; j < LCD_HEIGHT / 8; j++ )
    {
      for( i = 0; i < PILLER_WIDTH; i++ )
        {
          *screen++ |= *src++;
        }
      screen += ( SCREEN_WIDTH - PILLER_WIDTH );
    }
}

/************************************************
 *\
 *
 ***********************************************/
static void
disp_piller( void )
{
  int i, max = _game_state.piller_num;

  for( i = 0; i < max; i++ )
    {
      draw_piller( i );
    }
}


/************************************************
 *ړ
 *
 ***********************************************/
static void
move_piller( void )
{
  int i, max = _game_state.piller_num;
  UInt16 x;
  UInt16 or_bit;

  // EFCg
  if( --_game_state.piller_wait ) return;
  _game_state.piller_wait = DEF_PILLER_WAIT;

  for( i = 0; i < max; i++ )
    {
      if( ( (_piller[i].x >> 8) - (164 - 80 + 4) ) >= LCD_WIDTH + PILLER_WIDTH )// ʂ͂ݏoH
        {
          create_new_piller( i );//VZbg
          continue;
        }

      if( ++_piller[i].num == MAX_PILLER_SIZE && _game_state.mode == GM_PLAY )// @Ƃ̓蔻
        {
          if( is_crash( ( (UInt16) _piller[i].x >> 8 ) - ( 164 - 80 + 4 ) + SCREEN_WIDTH_BLANK - PILLER_WIDTH ) )
            {
               if( _piller[i].kind )
                {
                  // {[Qbg
                  _game_state.score += SCORE_BONUS;
                  set_flash_led();
                }
              else
                {
                  // Q[I[o[
                  _game_state.mode = GM_OVER;
                  _game_state.bomb_time = DEF_BOMB_TIME;
                  P2_OUT |= PIN_7;// LED
                  _bonus.time = 0;
                  _flash_led.time = 0;
                  set_bomb();
                  set_flash();
                }
              break;
            }
          create_new_piller( i );
          continue;
        }

      // ړ@1+1/16{ TC-100DASH!! AZu^^
      x = (UInt16) _piller[ i ].x - 0x8000;
      if( x & 0x8000 )
        or_bit = 0xf000;
      else
        or_bit = 0x0000;
      x = ( x >> 4 ) | or_bit;
      _piller[ i ].x += x;
    }
}

/************************************************
 *Ǝ@̓蔻
 *out: true @false ͂
 ***********************************************/
static Boolean
is_crash( int ex )
{
  Boolean hit = false;

  if( (UInt16) _machine.x - (UInt16) ex + HIT_MACHINE_WIDTH < HIT_MACHINE_WIDTH + HIT_PILLER_WIDTH +1 )
    {
      hit = true;
    }
  return hit;
}

/************************************************
 * n`
 *
 ***********************************************/
void
draw_horizon( void )
{
  int i;
  UInt16 *screen = (UInt16 *) _vram + (SCREEN_WIDTH + SCREEN_WIDTH_BLANK) /2;
  UInt16 data;

  data = _hr_data[ _horizon.no * 4 ];
  for( i = 0; i < LCD_WIDTH / 2 / 4; i++ )
    {
      *screen++ = data;//16bit * 4
      *screen++ = data;
      *screen++ = data;
      *screen++ = data;
    }
  screen += (SCREEN_WIDTH_BLANK * 2) /2;

  data = _hr_data[ _horizon.no * 4 + 1 ];
  for( i = 0; i < LCD_WIDTH / 2 / 4; i++ )
    {
      *screen++ = data;//16bit * 4
      *screen++ = data;
      *screen++ = data;
      *screen++ = data;
    }
  screen += (SCREEN_WIDTH_BLANK * 2) /2;

  data = _hr_data[ _horizon.no * 4 + 2 ];
  for( i = 0; i < LCD_WIDTH / 2 / 4; i++ )
    {
      *screen++ = data;//16bit * 4
      *screen++ = data;
      *screen++ = data;
      *screen++ = data;
    }
  screen += (SCREEN_WIDTH_BLANK * 2) /2;

  data = _hr_data[ _horizon.no * 4 + 3 ];
  for( i = 0; i < LCD_WIDTH / 2 / 4; i++ )
    {
      *screen++ = data;//16bit * 4
      *screen++ = data;
      *screen++ = data;
      *screen++ = data;
    }
}

/************************************************
 *nړ
 *
 ***********************************************/
void
update_horizon( void )
{
  if( !( --_horizon.wait) )
    {
      _horizon.no = ( _horizon.no + 1 ) & 3;
      _horizon.wait = DEF_HR_WAIT;
    }
}

/************************************************
 *xAbv
 *
 ***********************************************/
static void
level_up( void )
{
  if( _game_state.level >= MAX_LEVEL ) return;

  if( ! (--_game_state.lv_count) ) // xAbv
    {
      _game_state.lv_count = DEF_LV_COUNT;
      _game_state.level++;
      _game_state.piller_num ++;

      _lv_print.time = DEF_LV_TIME;
      _lv_print.interval = DEF_LEVEL_PRINT_INTERVAL;
      _lv_print.pos_x = START_LV_POS;
    }
}

/************************************************
 *@X^[gAj
 *
 ***********************************************/
static void
start_machine( void )
{
  if( --_start.wait ) return;
  _start.wait = DEF_START_WAIT;

  if( _machine.y == START_POSITION ) return;

  _machine.y--;
}

/************************************************
 *READY!!
 *
 ***********************************************/
static void
print_ready( void )
{
  if( ! (--_blink.time) )// _Ŏ
    {
      _game_state.mode = GM_INTRO;// Q[Jn

      _lv_print.time = DEF_LV_TIME;// x\
      _lv_print.interval = DEF_LEVEL_PRINT_INTERVAL;
      _lv_print.pos_x = START_LV_POS;

       _game_state.start_count = DEF_START_COUNT;
      return;
    }

  if( ! (--_blink.interval) )
    {
      _blink.blink_f = (int) _blink.blink_f  ^ 1;// tO]
      _blink.interval = DEF_BLINK_INTERVAL;
    }

  if( _blink.blink_f )
    {
      set_bitmap( 44, 2, FONT_READY_WIDTH, _ready_font );
    }
}

/************************************************
 *LEVEL\
 *
 ***********************************************/
static void
print_level( void )
{
  if( ! _lv_print.time ) return;

  _lv_print.time--;

  if( ! (--_lv_print.interval))
    {
      _lv_print.interval = DEF_LEVEL_PRINT_INTERVAL;

      if( _lv_print.pos_x != LAST_LV_POS )// x\Aj
        {
          _lv_print.pos_x +=2;
        }
    }

  set_bitmap( _lv_print.pos_x, 1, FONT_LEVEL_WIDTH, _level_font );

  if( _lv_print.time < LV_NUM_TIME )
    {
      disp_num( LV_PRINT_NUM_POS_X, 1, _game_state.level+1, 2 ); // x2
    }
}

/************************************************
 *PAUSE\
 *
 ***********************************************/
static void
print_pause( void )
{
  if( ! _game_state.pause_f ) return;

  if( ! (--_pause_print.interval) )
    {
      _pause_print.interval = DEF_PAUSE_PRINT_INTERVAL;

      if( _pause_print.pos_x != LAST_PAUSE_POS )// |[Y\Aj
        {
          _pause_print.pos_x -=2;
        }
      if( _info_print.pos_x != LAST_INFO_POS )// LEVEL SCORE HISCORE
        {
          _info_print.pos_x += 2;
        }
    }
  set_bitmap( _pause_print.pos_x, 4, FONT_PAUSE_WIDTH, _pause_font );
  // XRA\
  disp_score( _info_print.pos_x );
}

/************************************************
 *PAUSE
 *
 ***********************************************/
static void
pause( void )
{
  if( _key & KEY_RIGHT && _key & KEY_LEFT )
    {
       if( ! _game_state.pause_f )
        {
          _pause_print.interval = DEF_PAUSE_PRINT_INTERVAL;
          _pause_print.pos_x = START_PAUSE_POS;
          _info_print.pos_x = START_INFO_POS;

          _game_state.pause_f = true;
        }
    }
  else
    {
      _game_state.pause_f = false;
    }
}

/************************************************
 *XRAZ
 *
 ***********************************************/
static void
add_score( void )
{
  if( ! _game_state.pause_f )
    {
      _game_state.score++;
    }
}

/************************************************
 *XRA\
 *
 ***********************************************/
static void
disp_score( int pos )
{
  // SCORE
  set_bitmap( pos, 2, FONT_SCORE_WIDTH, _score_font );
  disp_num( pos + FONT_SCORE_WIDTH, 2, _game_state.score, SCORE_DIGIT );

  // HI SCORE
  set_bitmap( pos, 3, FONT_SCORE_WIDTH, _hi_font );
  disp_num( pos + FONT_SCORE_WIDTH, 3, _game_state.hiscore, SCORE_DIGIT );

  // LEVEL
  set_bitmap( pos, 1, FONT_LEVEL_WIDTH, _level_font );
  disp_num( pos + FONT_LEVEL_WIDTH, 1, _game_state.level + 1, 2 );
}

/************************************************
 *jZbg
 *
 ***********************************************/
static void
set_bomb( void )
{
  int i;

  for( i = 0; i < MAX_BOMB; i++ )
    {
      _bomb[i].interval = DEF_BOMB_INTERVAL;
      _bomb[i].num = 0;
      _bomb[i].x = _machine.x - 4 + RND( 16 );
      _bomb[i].y = _machine.y - 4 + RND( 8 );
    }
}

/************************************************
 *j\
 *
 ***********************************************/
static void
disp_bomb( void )
{
  int i;

  if( _game_state.bomb_time ) _game_state.bomb_time--;
  if( _game_state.bomb_time <= DEF_OVER_WAIT ) return;

    if( ! (--_bomb[0].interval) )
    {
      _bomb[0].interval = DEF_BOMB_INTERVAL;

      for( i = 0; i < MAX_BOMB; i++ )
        {
          _bomb[i].num++;
        }

      if( _bomb[0].num >= BOMB_ANIME_LIM )
       {
         set_bomb();// VjZbg
       }
    }

    for( i = 0; i < MAX_BOMB; i++ )
      {
        draw_bitmap( _bomb[i].x, _bomb[i].y, BOMB_WIDTH,  _bomb_data + _bomb[i].num * 8 );
      }
}

/************************************************
 *Q[I[o[\
 *
 ***********************************************/
static void
print_over( void )
{
  if( _game_state.bomb_time ) return;

  update_hiscore();// nCXRAXV

  set_bitmap( SCREEN_WIDTH_BLANK, 1, FONT_OVER_WIDTH, _over_font );
  set_bitmap( SCREEN_WIDTH_BLANK, 2, FONT_OVER_WIDTH, _over_font + 72 / 2 );

  // XRA
  set_bitmap( SCREEN_WIDTH_BLANK + OVER_SCORE_POS, 3, FONT_SCORE_WIDTH, _score_font );
  disp_num( SCREEN_WIDTH_BLANK + OVER_SCORE_POS + FONT_SCORE_WIDTH, 3, _game_state.score, SCORE_DIGIT );

  set_bitmap( SCREEN_WIDTH_BLANK + OVER_SCORE_POS, 4, FONT_SCORE_WIDTH, _hi_font );
  disp_num( SCREEN_WIDTH_BLANK + OVER_SCORE_POS + FONT_SCORE_WIDTH, 4, _game_state.hiscore, SCORE_DIGIT );

  if( _keyf & KEY_RIGHT || _keyf & KEY_LEFT)
    {
      _game_state.mode = GM_TITLE;
    }
}

/************************************************
 *nCXRAXV
 *
 ***********************************************/
static void
update_hiscore( void )
{
  if( _game_state.score > _game_state.hiscore )
    {
      _game_state.hiscore = _game_state.score;
    }
}

/************************************************
 *X^[g̃EFCgiCgj
 *
 ***********************************************/
static void
start_count( void )
{
  if( ! (--_game_state.start_count) )
    {
      _game_state.mode = GM_PLAY;
    }
}

/************************************************
 *tbV
 *
 ***********************************************/
static void
set_flash( void )
{
  _flash.blink_f = true;
  _flash.interval = DEF_FLASH_INTERVAL;
  _flash.time = DEF_FLASH_TIME;
}

static void
set_flash_led( void )
{
  _flash_led.blink_f = true;
  _flash_led.interval = DEF_FLASH_LED_INTERVAL;
  _flash_led.time = DEF_FLASH_LED_TIME;

  //bonus
  _bonus.blink_f = true;
  _bonus.interval = DEF_BONUS_INTERVAL;
  _bonus.time = DEF_BONUS_TIME;
}

/************************************************
 *ACeQbg LED_
 *
 ***********************************************/
static void
flash_led( void )
{
  if( ! _flash_led.time ) return;
  _flash_led.time--;

  if( ! (--_flash_led.interval) )
    {
      _flash_led.interval = DEF_FLASH_LED_INTERVAL;
      (UInt16) _flash_led.blink_f ^= 1;
    }

  // LED _
  if( _flash_led.blink_f )
    {
      P2_OUT &= ~PIN_7;
    }
  else
    {
      P2_OUT |= PIN_7;
    }
}

/************************************************
 *tbV
 *
 ***********************************************/

static void
flash( void )
{
  int i, j;
  UInt16 *screen = (UInt16 *) _vram + (SCREEN_WIDTH + SCREEN_WIDTH_BLANK) / 2;

  if( ! _flash.time ) return;
  _flash.time--;

  if( ! (--_flash.interval) )
    {
      _flash.interval = DEF_FLASH_INTERVAL;
      (UInt16) _flash.blink_f ^= 1;
    }

  if( ! _flash.blink_f ) return;

  for( j = 0; j < LCD_HEIGHT / 8; j++ )
    {
      for( i = 0; i < LCD_WIDTH / 2; i++ )
        {
          *screen++ ^= 0xffff;
        }
      screen += (SCREEN_WIDTH_BLANK * 2)/2;
    }
}

/************************************************
 *{[iX\
 *
 ***********************************************/
static void
bonus( void )
{
  if( ! _bonus.time ) return;
  _bonus.time--;

  if( ! (--_bonus.interval) )
    {
      _bonus.interval = DEF_BONUS_INTERVAL;
      (UInt16) _bonus.blink_f ^= 1;
    }

  // +1000pts
  if( _bonus.blink_f )
    {
      set_bitmap( SCREEN_WIDTH_BLANK, 1, FONT_BONUS_WIDTH, _bonus_font );
    }
}


/************************************************
 * nl\
 * x : xW ACg
 * y : yW 0-5
 * num F l
 * dF 
 ***********************************************/
static void
disp_num( int  x, int  y, UInt16 num, int d )
{
  int i, pos = (d-1) * DEF_NUM_WIDTH;


  for( i = 0; i < d; i++ )
    {
      set_bitmap( x+pos, y, DEF_NUM_WIDTH, _num_font + (num % 10) * FONT_NUM_WIDTH );
      num /=10;
      pos -= DEF_NUM_WIDTH;
    }
}

/************************************************
 *^Cg
 *
 ***********************************************/
void
init_title( void )
{
  //_
  _pre_title.interval = DEF_TITLE_INTERVAL;
  _pre_title.blink_f = true;
}
