 /***************************************************************************
 *                                                                          *
 * init.c                                                                   *
 * This is initialize driver program file for SVT17702.                   	*
 * History:  2008/03/26  Start.                                             *
 *                                                                          *
 ***************************************************************************/

 /* #include */
 #include "s1c17702.h"
 #include "init.h"

 /**********************************************************************
 *	controlPsc function.
 *  Prescaler run/stop control.
 *  param    :     control         0:stop   1:run
 *  return   :     noe
 *  history  :     2008/03/26   start
 ***********************************************************************/
 void controlPsc(unsigned char control){
 	  //Prescaler run/stop control(D0/0x4020)
 	  PSC_CTL_PRUN = control;
 }

 /**********************************************************************
 *	debugModePsc function.
 *  Prescaler run/stop setting in debug mode.
 *  param    :     debugMode       0:stop   1:run
 *  return   :     noe
 *  history  :     2008/03/26   start
 ***********************************************************************/
 void debugModePsc(unsigned char debugMode){
 	  //Prescaler run/stop setting in debug mode(D1/0x4020)
 	  PSC_CTL_PRUND = debugMode;
 }

 /************************************************************************
 * 	controlClgf unction.
 *  Clg run/stop control.
 *  param    :     control         0:stop   1:run
 *  return   :     none
 *  history  :     2008/03/26  start
 ***********************************************************************/
 void controlClg(unsigned char control){
      //PCLK enable (D[1:0]/0x5080)
      CLG_PCLK_PCKEN = control;
 }

 /************************************************************************
 * 	setClockGear function.
 *  Clock Gear setting.
 *  param    :     gear	       1/x
 *  return   :     none
 *  history  :     2008/03/26   start
 ***********************************************************************/
 void setClockGear(unsigned char gear){
 	//Clock Gear ratio select(D[1:0]/0x5081)
 	CLG_CCLK_CCLKGR = gear;
 }

 /************************************************************************
 * 	setMiscAccessCycle function.
 *  Flash / Sramc access cycle setting.
 *  param    :     char flashc 	0x0:2.5cycle,0x1:3.5cycle,0x2:4.5cycle,
 *                              0x3:5.5cycle,0x4:1.5cycle
 *                 char sramc	0x0:2cycle,0x1:3cycle,0x2:4cycle,0x3:5cycle
 *  return   :     none
 *  history  :     2008/03/26  start
 ***********************************************************************/
 void setMiscAccessCycle(unsigned char flashc,unsigned char sramc){
 	//Flashc read access cycle (D[2:0]/0x5320)
 	MISC_FL_FLCYC = flashc;
    //Sramc access cycle (D[9:8]/0x5320)
    MISC_FL_SRCYC = sramc;
 }

 /************************************************************************
 * 	debugModeMisc function.
 *  OSC1 peripheral contorl in Debug mode.
 *  param    :     debugMode       0:stop  1:run
 *  return   :     none
 *  history  :     2008/03/26  start
 ***********************************************************************/
 void debugModeMisc(unsigned char debugMode){
     //OSC1 peripheral run/stop contorl in Debug mode(D0/0x5322)
     MISC_OSC1_O1DBG = debugMode;
 }

 /************************************************************************
 * 	protectMisc function.
 *  Misc register write protect mode setting.
 *  param    :     protectMode       0x96:remove the wirte protect
 * 	 							  	 another:wirte protect
 *  return   :     none
 *  history  :     2008/03/26  start
 ***********************************************************************/
 void protectMisc(unsigned char protectMode){
     //Misc register wirte protect(D[15:0]/0x5324)
     MISC_PROT_PROT = protectMode;
 }

 /************************************************************************
 * 	setMiscIramSize function.
 *  Iram size setting.
 *  param    :     iramSize       0x0:12KB,0x1:8KB,0x2:4KB,0x3:2KB
 *  return   :     none
 *  history  :     2008/03/26  start
 ***********************************************************************/
 void setMiscIramSize(unsigned char iramSize){
     //Iram size select(D[1:0]/0x5326)
     MISC_IRAMSZ_IRAMSZ = iramSize;
 }

 /************************************************************************
 *	setMiscVecAddress function.
 *  Vector table address setting.
 *  param	  :		baseAddreL       0x0-0xff
 * 					baseAddreH       0x0-0xff
 *  return   :     none
 *  history  :     2008/03/26  start
 ***********************************************************************/
 void setMiscVecAddress(unsigned short baseAddreL,unsigned short baseAddreH){
     //Vector table base address low(D[15:8]/0x5328)
     MISC_TTBRL_TTBR = baseAddreL;
     //Vector table base address high(D[7:0]/0x532a)
     MISC_TTBRH_TTBR = baseAddreH;
 }

