 /***********************************************************************
 *                                                                      *
 * osc.c                                                                *
 * This is osc driver program file for SVT17702.                        *
 * History :  2008/03/28  Start.                                        *
 *                                                                      *
 ************************************************************************/
 
 #include "s1c17702.h"
 #include "osc.h"
 
/************************************************************************
 * 	 stopOsc1 function.
 *   Stop OSC1 
 *   param    :     void
 *   return   :     unsigned char OSC_STOP_NG
 *                       		  OSC_STOP_OK
 *   history  :     2008/03/28	start
 ***********************************************************************/
 unsigned char stopOsc1(void) {
	// Check CLKSRC (D[0]/0x5060)
	if(OSC_SRC_CLKSRC == OSC_SOURCE_OSC1) {
		return OSC_STOP_NG;
	}

 	// STOP OSC1 (D[1]/0x5061)
 	OSC_CTL_OSC1EN = 0x00;

 	return OSC_STOP_OK;
 }

/************************************************************************
 * 	 startOsc1 function.
 *   Start OSC1 
 *   param    :     void
 *   return   :     none
 *   history  :     2008/03/28	start
 ***********************************************************************/
 void startOsc1(void) {
	// Check OSC1EN (D[1]/0x5061)
 	if(OSC_CTL_OSC1EN == 0x01) {
 		return;
 	}

	// START OSC1 (D[1]/0x5061)
 	OSC_CTL_OSC1EN = 0x01;

 }

/************************************************************************
 *	 stopOsc3 function.
 *   Stop OSC3 
 *   param    :     void
 *   return   :     unsigned char OSC_STOP_NG
 *                       		  OSC_STOP_OK
 *   history  :     2008/03/28	start
 ***********************************************************************/
 unsigned char stopOsc3(void) {
	// Check CLKSRC (D[0]/0x5060)
	if(OSC_SRC_CLKSRC == OSC_SOURCE_HSCLK) {
		// Check HSCLKSEL (D[1]/0x5060)
		if(OSC_SRC_HSCLKSEL == OSC_SOURCE_OSC3){
			return OSC_STOP_NG;
		}
	}

 	// STOP OSC3 (D[0]/0x5061)
 	OSC_CTL_OSC3EN = 0x00;

	return OSC_STOP_OK;
 }

/************************************************************************
 * 	 startOsc3 function.
 *   Start OSC3 
 *   param    :     void
 *   return   :     none
 *   history  :     2008/03/28	start
 ***********************************************************************/
 void startOsc3(void) {
	// Check OSC3EN (D[0]/0x5061)
 	if(OSC_CTL_OSC3EN == 0x01) {
		return;
	}

	// START OSC3 (D[0]/0x5061)
 	OSC_CTL_OSC3EN = 0x01;

 }
 
/************************************************************************
 * 	 stopIosc function.
 *   Stop IOSC 
 *   param    :     void
 *   return   :     unsigned char OSC_STOP_NG
 *                       		  OSC_STOP_OK
 *   history  :     2008/03/28	start
 ***********************************************************************/
 unsigned char stopIosc(void) {
	// Check CLKSRC (D[0]/0x5060)
	if(OSC_SRC_CLKSRC == OSC_SOURCE_HSCLK) {
		// Check HSCLKSEL (D[1]/0x5060)
		if(OSC_SRC_HSCLKSEL == OSC_SOURCE_IOSC){
			return OSC_STOP_NG;
		}
	}

 	// STOP IOSC (D[2]/0x5061)
 	OSC_CTL_IOSCEN = 0x00;

	return OSC_STOP_OK;
 }

/************************************************************************
 * 	 startIosc function.
 *   Start IOSC 
 *   param    :     void
 *   return   :     none
 *   history  :     2008/03/28	start
 ***********************************************************************/
 void startIosc(void) {
	// Check IOSCEN (D[0]/0x5061)
 	if(OSC_CTL_IOSCEN == 0x01) {
		return;
	}

	// START IOSC (D[2]/0x5061)
 	OSC_CTL_IOSCEN = 0x01;

 }

/************************************************************************
 *	 changeSysClkHsclkTo1 function.
 *   Change clock HSCLK -> OSC1 
 *   param    :     void
 *   return   :     none
 *   history  :     2008/03/28	start
 ***********************************************************************/
 void changeSysClkHsclkTo1(void) { 	
 	startOsc1();								//OSC1 ON
 	
	// Check CLKSRC (D[0]/0x5060)
	if (OSC_SRC_CLKSRC == OSC_SOURCE_OSC1) {
		return;									//already OSC1
	}

	// Change CLKSRC (D[0]/0x5060)
	OSC_SRC_CLKSRC = OSC_SOURCE_OSC1;			//CLKSRC=OSC1
 }

/************************************************************************
 * 	 changeSysClkOsc1ToHsclk function.
 *   Change clock OSC1 -> HSCLK
 *   param    :     unsigned char selectClk    0:IOSC 1:OSC3
 *   return   :     none
 *   history  :     2008/03/28	start
 ***********************************************************************/
 void changeSysClkOsc1ToHsclk(unsigned char selectClk) {
 	startIosc();			//IOSC ON
 	startOsc3();			//OSC3 ON
 	
	if(selectClk == OSC_SOURCE_IOSC) {
		// Chanhe High speed (D[1]/0x5060)
		OSC_SRC_HSCLKSEL = OSC_SOURCE_IOSC;		//IOSC select	
	}
	else if(selectClk == OSC_SOURCE_OSC3) {
		// Chanhe High speed (D[1]/0x5060)
		OSC_SRC_HSCLKSEL = OSC_SOURCE_OSC3;		//OSC3 select	
	}
	else{
		return;
	}
	
	// Check CLKSRC (D[0]/0x5060)
	if (OSC_SRC_CLKSRC == OSC_SOURCE_HSCLK) {
		return;									//already HSCLK
	}	
	// Change CLKSRC (D[0]/0x5060)
	OSC_SRC_CLKSRC = OSC_SOURCE_HSCLK;			//CLKSRC=HSCLK
 }
 
 /************************************************************************
 *	 changeSysClkIoscTo3 function. 
 *   Change clock IOSC -> OSC3
 *   param    :     void
 *   return   :     none
 *   history  :     2008/03/28	start
 ***********************************************************************/
 void changeSysClkIoscTo3(void) {
 	// Check CLKSRC (D[0]/0x5060)
 	if(OSC_SRC_CLKSRC != OSC_SOURCE_HSCLK) { 	//not HSCLK
 		return;
 	}
 	
	startOsc3();								//OSC3 ON
	
	// Check CLKSRC (D[0]/0x5060)
	if (OSC_SRC_HSCLKSEL == OSC_SOURCE_OSC3) {  //already OSC3
		return;
	}

	// Chanhe High speed (D[1]/0x5060)
	OSC_SRC_HSCLKSEL = OSC_SOURCE_OSC3;			//OSC3 select

 }

 /************************************************************************
 *	 changeSysClkOsc3ToIosc function. 
 *   Change clock OSC3 -> IOSC
 *   param    :     void
 *   return   :     none
 *   history  :     2008/03/28	start
 ***********************************************************************/
 void changeSysClkOsc3ToIosc(void) {
 	// Check CLKSRC (D[0]/0x5060)
 	if(OSC_SRC_CLKSRC != OSC_SOURCE_HSCLK) { 	//not HSCLK
 		return;
 	}
 	
 	startIosc();								//IOSC ON

	// Check CLKSRC (D[0]/0x5060)
	if (OSC_SRC_HSCLKSEL == OSC_SOURCE_IOSC) {  //already IOSC
		return;
	}

	// Chanhe High speed (D[1]/0x5060)
	OSC_SRC_HSCLKSEL = OSC_SOURCE_IOSC;			//IOSC select

 }

/************************************************************************
 * 	 initFoutH function.
 *   Initialize FOUTH.
 *   param    :     unsigned char clkDivRate :FOUT3_CLK_DIV_RATE_1,FOUT3_CLK_DIV_RATE_2,
 * 									 		  FOUT3_CLK_DIV_RATE_4
 *   return   :     none
 *   history  :     2008/03/28	start
 ***********************************************************************/
 void initFoutH(unsigned char clkDivRate) {

	// FOUT3 clock division ratio select
	//(D[2:3]/0x5064)
	OSC_FOUT_FOUTHD = clkDivRate;	

	// P30=FOUT (D[0]/0x52A3)
	P3_PMUX_P30MUX = 0x01;
 }


/************************************************************************
 * 	 startFoutH function.
 *   start FOUTH
 *   param    :     unsigned char selectClk    0:IOSC 1:OSC3
 *   return   :     none
 *   history  :     2008/03/028	start
 ***********************************************************************/
 void startFoutH(unsigned char selectClk) {
	if(selectClk == OSC_SOURCE_IOSC) {
		startIosc();							//IOSC ON
	}
	else if(selectClk == OSC_SOURCE_OSC3) {
		startOsc3();							//OSC3 ON
	}
	else{
		return;
	}
	
	// FOUTH output enable (D[1]/0x5064)
	OSC_FOUT_FOUTHE = 0x01;

 }

/************************************************************************
 * 	 stopFoutH function.
 *   Stop FOUTH
 *   param    :     void
 *   return   :     none
 *   history  :     2008/03/28	start
 ***********************************************************************/
 void stopFoutH(void) {

	// FOUT3 output disable (D[1]/0x5063)
	OSC_FOUT_FOUTHE = 0x00;

 }

/************************************************************************
 * 	 initFout1 function.
 *   Initialize Fout3
 *   param    :     void
 *   return   :     none
 *   history  :     2008/03/28	start
 ***********************************************************************/
 void initFout1(void) {
	// P13=FOUT (D[3]/0x52A1)
	P1_PMUX_P13MUX = 0x01;
 }

/************************************************************************
 * 	 startFout1 function.
 *   start Fout1
 *   param    :     void
 *   return   :     none
 *   history  :     2008/03/28	start
 ***********************************************************************/
 void startFout1(void) {
	startOsc1();
	
	// FOUT1 output enable (D[0]/0x5064)
	OSC_FOUT_FOUT1E = 0x01;

 }

/************************************************************************
 * 	 stopFout1 function.
 *   Stop Fout1
 *   param    :     void
 *   return   :     none
 *   history  :     2008/03/28	start
 ***********************************************************************/
 void stopFout1(void) {
	// FOUT1 output disable (D[0]/0x5064)
	OSC_FOUT_FOUT1E = 0x00;

 }
